package com.dxhy.core.job.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.dxhy.common.utils.PageUtils;
import com.dxhy.core.job.entity.ScheduleJobEntity;

import java.util.List;
import java.util.Map;

/**
 * 定时任务
 *
 * @author user
 */

public interface ScheduleJobService extends IService<ScheduleJobEntity> {
    /**
     * 翻页查询
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param params
     * @return com.dxhy.common.utils.PageUtils
     */
    PageUtils queryPage(Map<String, Object> params);

    /**
     * 保存定时任务
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param scheduleJob
     * @return void
     */
    void saveJob(ScheduleJobEntity scheduleJob);

    /**
     * 更新定时任务
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param scheduleJob
     * @return void
     */
    void update(ScheduleJobEntity scheduleJob);

    /**
     * 批量删除定时任务
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param jobId
     * @return void
     */
    void deleteBatch(Long jobId);

    /**
     * 批量更新定时任务状态
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param jobId
     * @param status
     * @return int
     */
    int updateBatch(Long jobId, int status);

    /**
     * 立即执行
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param jobId
     * @return void
     */
    void run(Long jobId);

    /**
     * 暂停运行
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param jobId
     * @return void
     */
    void pause(Long jobId);

    /**
     * 恢复运行
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param jobId
     * @return void
     */
    void resume(Long jobId);

    /**
     * 根据id查询
     *
     * @author 陈刚扬
     * @date 2020/10/30
     * @param jobId
     * @return com.dxhy.core.job.entity.ScheduleJobEntity
     */
    ScheduleJobEntity queryById(Long jobId);

    ScheduleJobEntity queryByBeanName(String beanName, String methodName);
    /**
     * @Description 通过定时任务类名和方法名执行任务。数组形式，顺序执行，每个任务之间间隔15秒
     * @param jobNames  beanName 名   methodName方法名
     * @Return org.springframework.http.ResponseEntity<?>
     * @Author wangruwei
     * @Date 2024/7/19 11:08
     **/
    void doScheJobByBeanNameMethods(List<Map<String, String>> jobNames);

    void timedTaskRestore();
}
