package com.dxhy.core.thread;


import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.api.model.config.Business;
import com.dxhy.api.model.excelExport.TDxExcelExportlog;
import com.dxhy.common.rabbitmq.RabbitMqSender;
import com.dxhy.common.service.impl.TabulateRedisServiceImpl;
import com.dxhy.common.util.ExcelPoiUtil;
import com.dxhy.common.util.ExceptionUtil;
import com.dxhy.common.util.StringHelper;
import com.dxhy.common.utils.DateUtils;
import com.dxhy.core.feign.IFeginWebsocket;
import com.dxhy.core.model.FpRzRespVO;
import com.dxhy.core.service.export.RzFpsgrzService;
import com.dxhy.core.service.singletable.IExcelExportlogService;
import com.dxhy.core.util.JsonResponseUtils;
import com.dxhy.oss.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

/**
 * @author 发票认证导出
 */

@SuppressWarnings("AlibabaMethodTooLong")
@Slf4j

public class RzFprzExportThread extends BaseThread {

    /**
     * 默认要导出第几页的数据
     */
    private final int default_curr = 1;
    /**
     * 默认每页导出多少条数据
     */
    private final int default_size = 5000;
    /**
     * 文件名前缀
     */
    private final String fileNamePrefix = "rzfpsgrzList";
    /**
     * 文件名后缀
     */
    private final String fileNameSurfix = ".xlsx";
    /**
     * 文件名中间分隔符
     */
    private final String fileSplit = "_";
    /**
     * excel的标题
     */
    private final String title = "手工认证";
    /**
     * 导出excel的列名
     */

    private final IExcelExportlogService exportLogService;

    private final List<TDxExcelExportlog> exportLogs;

    private final RzFpsgrzService rzFpsgrzService;

    private final FileService fileService;

    private final RabbitMqSender sender;
    private final TabulateRedisServiceImpl tabulateRedisService;

    public RzFprzExportThread(IExcelExportlogService exportLogService, List<TDxExcelExportlog> exportLogs,
                              RzFpsgrzService rzFpsgrzService, FileService fileService, RabbitMqSender sender, IFeginWebsocket feginWebsocket,
                              TabulateRedisServiceImpl tabulateRedisService) {

        this.exportLogService = exportLogService;
        this.exportLogs = exportLogs;
        this.rzFpsgrzService = rzFpsgrzService;
        this.fileService = fileService;
        this.sender = sender;
        this.feginWebsocket = feginWebsocket;
        this.tabulateRedisService = tabulateRedisService;
    }

    private final IFeginWebsocket feginWebsocket;

    @Override
    public void run() {
        for (TDxExcelExportlog exportLog : exportLogs) {
            // 日志表主键
            Long id = exportLog.getId();
            String userid = exportLog.getUserAccount();
            String userName = exportLog.getUserName();
            // 导出条件
            String condition = exportLog.getConditions();
            // 数据源名称
            String dbName = exportLog.getDbName();
            if (StringHelper.isBlank(dbName)) {
                dbName = "";
            }
            String createDate = DateUtils.dateToStrLong(exportLog.getCreateDate());

            JSONObject msg = new JSONObject();

            // 捕获异常，防止某一条日志导出出错，影响其它日志的导出
            try {
                exportLogService.updateStart(id, dbName);

                // 应该保存的参数
                JSONObject map = JSONObject.parseObject(condition);
                String gfsh = (String)map.get("gfsh");
                String invoiceType = (String)map.get("invoiceType");
                String invoiceSource = (String)map.get("invoiceSource");
                String kpksrq = (String)map.get("kpksrq");
                String kpjsrq = (String)map.get("kpjsrq");
                String qszt = (String)map.get("qszt");
                String qsfs = (String)map.get("qsfs");
                String businessCode = (String)map.get("businessCode");
                String invoiceNo = (String)map.get("invoiceNo");
                String qsksrq = (String)map.get("qsksrq");
                String qsjsrq = (String)map.get("qsjsrq");
                String xfmc = (String)map.get("xfmc");
                String position = (String)map.get("position");
                String fpzt = (String)map.get("invoiceStatus");
                String sfkrz = (String)map.get("sfkrz");
                String qsr = (String)map.get("qsr");
                String company = (String)map.get("company");
                String menuId = (String)map.get("menuId");

                JSONObject jsonObject = tabulateRedisService.getTabulateExcel(company, menuId);

                Map<String, Object> pramsMap = new HashMap<>(18);
                pramsMap.put("company", company);
                pramsMap.put("menuId", menuId);
                List<Business> businesses = (List<Business>)map.get("business");
                pramsMap.put("business", businesses);
                pramsMap.put("dbName", dbName);
                List<String> gfshList = new ArrayList<>();
                if (!StringHelper.isBlank(gfsh)) {
                    gfshList = StringHelper.stringToList(gfsh);
                    pramsMap.put("gfsh", gfshList);
                } else {
                    pramsMap.put("gfsh", gfshList);
                }
                pramsMap.put("payment", map.get("payment"));
                pramsMap.put("paymentStatus", map.get("paymentStatus"));
                pramsMap.put("paymentDateStart", map.get("paymentDateStart"));
                pramsMap.put("paymentDateEnd", map.get("paymentDateEnd"));
                pramsMap.put("settlementNo", map.get("settlementNo"));
                pramsMap.put("voucherNumber", map.get("voucherNumber"));
                String inAccountStatus = (String)map.get("inAccountStatus");

                if (sfkrz != null) {
                    if (!"99".equals(sfkrz)) {
                        pramsMap.put("sfkrz", sfkrz);
                    }
                }
                if (invoiceSource != null) {
                    if (!"99".equals(invoiceSource)) {
                        pramsMap.put("invoiceSource", invoiceSource);
                    }
                }

                if (!StringUtils.isBlank(invoiceType)) {
                    if (!"99".equals(invoiceType)) {
                        pramsMap.put("invoiceType", invoiceType);
                    }
                }
                if (StringUtils.isNotBlank(fpzt)) {
                    pramsMap.put("fpzt", fpzt);
                }
                pramsMap.put("businessCode", "99");
                if (!StringUtils.isBlank(qszt)) {
                    if (!"99".equals(qszt)) {
                        pramsMap.put("qszt", qszt);
                        if ("1".equals(qszt)) {
                            if (!StringUtils.isBlank(qsksrq)) {
                                pramsMap.put("qsksrq", qsksrq);
                            }
                            if (!StringUtils.isBlank(qsjsrq)) {
                                pramsMap.put("qsjsrq", qsjsrq);
                            }
                            if (!StringUtils.isBlank(qsfs)) {
                                // 表示全部
                                if (!"99".equals(qsfs)) {
                                    pramsMap.put("qsfs", qsfs);
                                }
                            }
                            pramsMap.put("businessCode", businessCode);
                        }
                    }
                }
                if (!StringUtils.isBlank(kpksrq)) {
                    pramsMap.put("kpksrq", kpksrq);
                }
                if (!StringUtils.isBlank(kpjsrq)) {
                    pramsMap.put("kpjsrq", kpjsrq);
                }

                if (!StringUtils.isBlank(position)) {
                    pramsMap.put("username", userName);
                    pramsMap.put("position", position);
                }

                if (!StringUtils.isBlank(invoiceNo)) {
                    if(invoiceNo.length()==8){
                        pramsMap.put("invoiceNo", invoiceNo);
                    }else {
                        pramsMap.put("eleInvoiceNo", invoiceNo);
                    }
                }
                if (!StringUtils.isBlank(xfmc)) {
                    pramsMap.put("xfmc", xfmc);
                }
                if (!StringUtils.isBlank(qsr)) {
                    pramsMap.put("qsr", qsr);
                }
                if (StringUtils.isNotBlank(inAccountStatus)) {
                    pramsMap.put("inAccountStatus", inAccountStatus);
                }




                // 导出文件名
                StringBuilder ftpFileName = new StringBuilder();
                ftpFileName.append(userid);
                ftpFileName.append(fileSplit);
                ftpFileName.append(fileNamePrefix);
                ftpFileName.append(fileSplit);
                ftpFileName.append(DateUtils.getStringDateShort());
                ftpFileName.append(fileSplit);
                ftpFileName.append((new Date()).getTime());
                ftpFileName.append(fileNameSurfix);

                // 本地文件存储路径
                StringBuilder excelFile = new StringBuilder();
                excelFile.append(fileService.getLocalPathDefault());
                excelFile.append(ftpFileName.toString());

                // 递归导出数据到excel
                this.cicleExport(pramsMap, default_curr, excelFile.toString(), jsonObject);

                // FTP文件存储路径
                String[] toDay = DateUtil.today().split("-");
                String ftpFilePath = fileService.getPathPrefix() + userName + "/" + "excelFile/" + toDay[0] + "/"
                    + toDay[1] + "/" + toDay[2] + "/" + ftpFileName.toString();
                // 文件上传ftp
                fileService.uploadFile(ftpFilePath, excelFile.toString());

                // 设置成功消息内容
                msg.put("title", super.getSuccTitle(title));
                msg.put("content", super.getSuccContent(createDate));
                msg.put("url", super.getUrl(id));
                msg.put("useraccount", userName);

                // 记录导出完成状态
                exportLogService.updateSucc(id, ftpFilePath, dbName);

            } catch (Exception e) {
                String errmsg = ExceptionUtil.getExceptionDeteil(e, 4000);
                // 设置失败消息内容
                msg.put("title", super.getFailTitle(title));
                msg.put("content", super.getFailContent(createDate, errmsg));
                msg.put("url", "");
                msg.put("useraccount", userName);

                log.error("", e);
                // 记录导出出错状态
                exportLogService.updateFail(id, errmsg, dbName);
            } finally {
                // 发送导出消息到mq
                String responseResult = JsonResponseUtils.responseResult(msg);
                sender.sendToMsg(responseResult);

                // 申请成功后像websocket发送申请信息
                feginWebsocket.setMessageToWebsocket(userName);
            }
        }
    }

    /**
     * 递归导出数据到excel文件，防止jvm oom
     *
     * @param pramsMap
     *            查询数据参数集合
     * @param curr
     *            当前第几页
     * @param excelFile
     *            导出的文件
     * @throws Exception
     *
     */

    private boolean cicleExport(Map<String, Object> pramsMap, int curr, String excelFile, JSONObject jsonObject)
        throws Exception {
        // 查询指定页数的数据
        JSONObject pageData = rzFpsgrzService.selectByPramsMap(pramsMap, curr, default_size);
        boolean hasNext = pageData.getBooleanValue("hasNext");
        List<FpRzRespVO> dataList = (List<FpRzRespVO>)pageData.get("datalist");
        boolean firstPage = false;
        if (curr == default_curr) {
            firstPage = true;
        }

        // 数据不是最后一页
        if (hasNext) {
            int nextPage = pageData.getIntValue("nextPage");
            // 写入当前页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, false);
            // 更新页数
            return cicleExport(pramsMap, nextPage, excelFile, jsonObject);
        } else {
            // 写入最后一页数据
            ExcelPoiUtil.exportListArrayToExcel1(excelFile, title, jsonObject, dataList, firstPage, true);
            return hasNext;
        }

    }
}
