package com.dxhy.auth.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.dxhy.api.model.bigB.BigBOrgInfoEntity;
import com.dxhy.api.model.bigB.BigBUserInfoEntity;
import com.dxhy.api.model.config.Business;
import com.dxhy.api.model.config.Tax;
import com.dxhy.api.model.config.UserInfo;
import com.dxhy.api.model.config.sys.SysGroupConfigEntity;
import com.dxhy.auth.config.UserCenterConfig;
import com.dxhy.auth.dao.SysGroupConfigAuthDao;
import com.dxhy.auth.service.UserCenterService;
import com.dxhy.auth.util.UserInfoRedisUtils;
import com.dxhy.common.constant.ConfigureConstant;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.exception.BaseException;
import com.dxhy.common.util.HttpUtils;
import com.dxhy.common.util.JsonUtils;
import com.dxhy.common.utils.BaseContextHandler;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Allen
 * @version 1.0.0
 * @ClassName UserCenterServiceImpl.java
 * @Description 用户模块实现
 * @createTime 2023年06月29日 14:23:00
 */
@Component
@Slf4j
public class UserCenterServiceImpl implements UserCenterService {

    @Autowired
    private UserCenterConfig userCenterConfig;

    @Autowired
    private SysGroupConfigAuthDao sysGroupConfigAuthDao;

    @Resource
    private UserInfoRedisUtils userInfoRedisUtils;

    @Autowired
    private Environment environment;

    @Override
    public boolean checkToken(String requestToken) {
        Map<String,String> headers = getHeader(requestToken);
        log.info("请求的headers:{}",JSON.toJSONString(headers));
        log.info("请求的地址:{}",userCenterConfig.checkToken());
        String bigBResp = HttpUtils.doPostWithHeader(userCenterConfig.checkToken(), null, headers);
        //String bigBResp = "{\"code\":\"0000\",\"message\":\"已登录\"}";
        //log.info("第一个要改的@@@@@@:{}",bigBResp);
        String code = JSON.parseObject(bigBResp).getString(ConfigureConstant.CODE);
        if(Arrays.asList(environment.getActiveProfiles()).contains("dev")){
            //开发环境，直接返回true
            return true;
        }
        if (ConfigureConstant.SECCESS_CODE.equals(code)) {
            log.debug("token有效");
            return true;
        } else {
            log.debug("token失效");
            return false;
        }

//        return true;
    }

    private Map<String, String> getHeader(String requestToken) {
        Map<String, String> headers = new HashMap<>();
        headers.put(ConfigureConstant.BIGB_TOKEN_HEADER_NAME, ConfigureConstant.DXHY_SSO_SESSION_ID+"="+requestToken);
        return headers;
    }


    @Override
    public String getUserBasicInfo(String requestToken) {
        Map<String,String> headers = getHeader(requestToken);
        String response = "";
        if(Arrays.asList(environment.getActiveProfiles()).contains("dev")){
            //开发环境，直接返回true
            response = "{\"code\":\"0000\",\"data\":{\"userId\":11002370091,\"username\":\"wrw\",\"email\":null,\"mobile\":\"\",\"status\":1,\"createTime\":\"2023-09-07 16:11:57\",\"number\":null,\"salt\":null,\"name\":\"王汝伟\",\"companyCode\":\"123456789654123\",\"companyName\":\"进项测试集团勿动\",\"dept\":{\"deptId\":155354,\"parentId\":0,\"name\":\"进项测试集团勿动\",\"code\":\"155354A\",\"level\":1,\"taxpayerCode\":\"123456789654123\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:02:41\",\"deptType\":1,\"createUser\":11002370089,\"contactName\":\"zhangzq\",\"contactPhone\":\"13681594654\",\"contactEmail\":null,\"deptSname\":\"进项测试集团勿动\",\"enterpriseNumbers\":\"yEJdZHWNGG\",\"ompDeptId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"0\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},\"dataPerm\":[155362,155363,1,155360,155361,155364,155787,155788,155795,155760,155761,155354,155355,155358,155359,155356,155804,155357],\"deptList\":[{\"deptId\":155362,\"parentId\":155354,\"name\":\"测试部\",\"code\":\"155354A155362A\",\"level\":2,\"taxpayerCode\":null,\"taxpayerName\":null,\"taxpayerAddress\":\"成都市金牛区金府路44\",\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:17:39\",\"deptType\":4,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"测试部\",\"enterpriseNumbers\":\"QA\",\"ompDeptId\":\"9465127600cd49b49f171680c720d33d\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155363,\"parentId\":155354,\"name\":\"安全部\",\"code\":\"155354A155363A\",\"level\":2,\"taxpayerCode\":null,\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:18:04\",\"deptType\":4,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"安全部\",\"enterpriseNumbers\":\"AQB\",\"ompDeptId\":\"d95e505cfd8d4d588e70dad296aa25e2\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":1,\"parentId\":155354,\"name\":\"大象\",\"code\":\"155354A155365A\",\"level\":2,\"taxpayerCode\":null,\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:19:00\",\"deptType\":5,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"大象\",\"enterpriseNumbers\":\"ELE\",\"ompDeptId\":\"a6c13ad88a6843cc9eee942566a3016d\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155360,\"parentId\":155354,\"name\":\"理想智行汽车销售服务（上海）有限公司\",\"code\":\"155354A155360A\",\"level\":2,\"taxpayerCode\":\"91310115MA1K4BBH8A\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:16:02\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"理想\",\"enterpriseNumbers\":\"LX\",\"ompDeptId\":\"0ba3440148e84d3396878bc8a74a3ae8\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155361,\"parentId\":155354,\"name\":\"林德（中国）投资有限公司\",\"code\":\"155354A155361A\",\"level\":2,\"taxpayerCode\":\"913100007109388104\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:16:29\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"林德\",\"enterpriseNumbers\":\"LD\",\"ompDeptId\":\"bce93a81ea224b2da9a0e435a1d6a826\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155364,\"parentId\":155354,\"name\":\"老虎\",\"code\":\"155354A155364A\",\"level\":2,\"taxpayerCode\":null,\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:18:39\",\"deptType\":5,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"老虎\",\"enterpriseNumbers\":\"TIGER\",\"ompDeptId\":\"0da557aebdbe4c689dd258e342365c16\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155787,\"parentId\":155354,\"name\":\"腾讯科技（深圳）有限公司\",\"code\":\"155354A155787A\",\"level\":2,\"taxpayerCode\":\"9144030071526726XG\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-10-24 16:07:23\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"腾讯\",\"enterpriseNumbers\":\"TX\",\"ompDeptId\":\"10c171fe02234e65998f3666cd1956ab\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155788,\"parentId\":155354,\"name\":\"北汽福田汽车股份有限公司\",\"code\":\"155354A155788A\",\"level\":2,\"taxpayerCode\":\"911100001012029043\",\"taxpayerName\":null,\"taxpayerAddress\":\"北京市昌平区沙河镇沙阳路老牛湾村北\",\"taxpayerPhone\":null,\"taxpayerBank\":\"建行北京市沙河支行\",\"taxpayerAccount\":\"11001016000056038212\",\"areaId\":null,\"createTime\":\"2023-10-24 20:34:10\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"北汽福田\",\"enterpriseNumbers\":\"FT\",\"ompDeptId\":\"b08e2b557c324ee3853eed2c6aca185e\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155795,\"parentId\":155354,\"name\":\"乐企农产品测试专用\",\"code\":\"155354A155795A\",\"level\":2,\"taxpayerCode\":\"91110116MACP7E4N1E\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-10-26 15:56:29\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"乐企农产品测试专用\",\"enterpriseNumbers\":\"NCPtest\",\"ompDeptId\":\"d54b0ef0abae46c9b9cb7788db47cb63\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155760,\"parentId\":155354,\"name\":\"测试1\",\"code\":\"155354A155760A\",\"level\":2,\"taxpayerCode\":\"91150121MA0R6RDC9A\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-10-12 16:51:04\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"22\",\"enterpriseNumbers\":\"211\",\"ompDeptId\":\"3ef03dc5f7454dbd99df05929200b70a\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155761,\"parentId\":155354,\"name\":\"测试2\",\"code\":\"155354A155761A\",\"level\":2,\"taxpayerCode\":\"91150000114124263Y\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-10-12 16:51:27\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"dfd\",\"enterpriseNumbers\":\"xcvsdf\",\"ompDeptId\":\"b4c91c08b972499093caf6ccae2a53cf\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155354,\"parentId\":0,\"name\":\"进项测试集团勿动\",\"code\":\"155354A\",\"level\":1,\"taxpayerCode\":\"123456789654123\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:02:41\",\"deptType\":1,\"createUser\":11002370089,\"contactName\":\"zhangzq\",\"contactPhone\":\"13681594654\",\"contactEmail\":null,\"deptSname\":\"进项测试集团勿动\",\"enterpriseNumbers\":\"yEJdZHWNGG\",\"ompDeptId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"0\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155355,\"parentId\":155354,\"name\":\"大象慧云信息技术有限公司\",\"code\":\"155354A155355A\",\"level\":2,\"taxpayerCode\":\"91110108MA004CPN95\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:07:41\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"大象\",\"enterpriseNumbers\":\"DX\",\"ompDeptId\":\"2952bd47627d49fa8d9ac40da5bc58c9\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155358,\"parentId\":155354,\"name\":\"江苏京东信息技术有限公司\",\"code\":\"155354A155358A\",\"level\":2,\"taxpayerCode\":\"91321311690769465E\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:14:34\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"91321311690769465E\",\"enterpriseNumbers\":\"JDXX\",\"ompDeptId\":\"a5d578b9dec145d2947928010b111a97\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155359,\"parentId\":155354,\"name\":\"北京京东世纪贸易有限公司\",\"code\":\"155354A155359A\",\"level\":2,\"taxpayerCode\":\"911103026605015136\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:15:19\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"JDSJ\",\"enterpriseNumbers\":\"JDSJ\",\"ompDeptId\":\"1180605bb3d14f8cbc6a14829c246b18\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155356,\"parentId\":155354,\"name\":\"进项测试企业公司\",\"code\":\"155354A155356A\",\"level\":2,\"taxpayerCode\":\"9111112222333344DD\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:10:31\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"进项测试企业\",\"enterpriseNumbers\":\"JXCS\",\"ompDeptId\":\"85f8d73b47964b5c96a9e6b20edebece\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155804,\"parentId\":155354,\"name\":\"进项代扣代缴\",\"code\":\"155354A155804A\",\"level\":2,\"taxpayerCode\":\"91310116MA1JCJWQXF\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-12-11 14:50:14\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"代扣代缴\",\"enterpriseNumbers\":\"dkdj\",\"ompDeptId\":\"49cf0c84889f41ae949be1a7ddc5f825\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"},{\"deptId\":155357,\"parentId\":155354,\"name\":\"耐克商业（中国）有限公司\",\"code\":\"155354A155356A155357A\",\"level\":2,\"taxpayerCode\":\"91310000596407652N\",\"taxpayerName\":null,\"taxpayerAddress\":null,\"taxpayerPhone\":null,\"taxpayerBank\":null,\"taxpayerAccount\":null,\"areaId\":null,\"createTime\":\"2023-09-07 10:13:20\",\"deptType\":2,\"createUser\":11002370089,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"耐克\",\"enterpriseNumbers\":\"NIKE\",\"ompDeptId\":\"db2a3e0cd6bd4192baedc3665e9ad2e4\",\"tenantId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"ompParentId\":\"d7f72d5efba047d7b79b9dfa72cbb438\",\"sourceId\":\"0f93e901354e4472967274d0b2739814\"}],\"plugininfo\":null,\"version\":\"f1bc70d336db46cb944c21aa3acf6781\",\"expireMinite\":300,\"expireFreshTime\":1702374224162,\"loginCount\":0,\"lastLoginTime\":\"2023-12-12 16:22:21\",\"boradAuth\":0},\"message\":\"success\"}";
        }else{

            //response = HttpUtils.doPostWithHeader(userCenterConfig.getBasicUserInfourl(), null, headers);
            response = userInfoRedisUtils.getRedisUserInfo(requestToken,userCenterConfig.getBasicUserInfourl(),headers);
            //response = "{\"code\":\"0000\",\"data\":{\"userId\":114,\"username\":\"xielina\",\"email\":\"xli0729@163.com\",\"mobile\":\"13146019908\",\"status\":1,\"createTime\":\"2023-09-15 15:49:51\",\"number\":null,\"salt\":null,\"name\":\"谢丽娜\",\"companyCode\":\"111122223333444\",\"companyName\":\"测试集团\",\"dept\":{\"deptId\":1,\"parentId\":0,\"name\":\"测试集团\",\"code\":\"1A\",\"level\":1,\"taxpayerCode\":\"111122223333444\",\"taxpayerName\":null,\"taxpayerAddress\":\"1\",\"taxpayerPhone\":\"\",\"taxpayerBank\":\"\",\"taxpayerAccount\":\"\",\"areaId\":11,\"createTime\":\"2019-03-08 10:48:35\",\"deptType\":1,\"createUser\":null,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"测试集团\",\"enterpriseNumbers\":\"111122223333444\",\"ompDeptId\":null,\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},\"dataPerm\":[1,34,35,36,37,38,39,40,41,42,43,44,45],\"deptList\":[{\"deptId\":1,\"parentId\":0,\"name\":\"测试集团\",\"code\":\"1A\",\"level\":1,\"taxpayerCode\":\"111122223333444\",\"taxpayerName\":null,\"taxpayerAddress\":\"1\",\"taxpayerPhone\":\"\",\"taxpayerBank\":\"\",\"taxpayerAccount\":\"\",\"areaId\":11,\"createTime\":\"2019-03-08 10:48:35\",\"deptType\":1,\"createUser\":null,\"contactName\":null,\"contactPhone\":null,\"contactEmail\":null,\"deptSname\":\"测试集团\",\"enterpriseNumbers\":\"111122223333444\",\"ompDeptId\":null,\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":34,\"parentId\":1,\"name\":\"91150121MA0R6RDC9A\",\"code\":\"1A34A\",\"level\":2,\"taxpayerCode\":\"91150121MA0R6RDC9A\",\"taxpayerName\":null,\"taxpayerAddress\":\"广州市番禺区大学城外环东路132号北学院楼B栋305室\",\"taxpayerPhone\":\"02084110129\",\"taxpayerBank\":\"00000000000\",\"taxpayerAccount\":\"123456778899\",\"areaId\":11,\"createTime\":\"2023-09-14 10:15:38\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"911501\",\"enterpriseNumbers\":\"91150121MA0R6RDC9A\",\"ompDeptId\":\"862122945913438208\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":35,\"parentId\":1,\"name\":\"91110000101174712L\",\"code\":\"1A35A\",\"level\":2,\"taxpayerCode\":\"91110000101174712L\",\"taxpayerName\":null,\"taxpayerAddress\":\"广州市番禺区大学城外环东路132号北学院楼B栋305室\",\"taxpayerPhone\":\"02084110129\",\"taxpayerBank\":\"销项方开户银行\",\"taxpayerAccount\":\"0000000000000000000\",\"areaId\":null,\"createTime\":\"2023-09-14 14:40:39\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"91110000101174712L\",\"enterpriseNumbers\":\"91110000101174712L\",\"ompDeptId\":\"862189639465844736\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":36,\"parentId\":1,\"name\":\"91110108MA004CPN95\",\"code\":\"1A36A\",\"level\":2,\"taxpayerCode\":\"91110108MA004CPN95\",\"taxpayerName\":null,\"taxpayerAddress\":\"广州市番禺区大学城外环东路132号北学院楼B栋305室\",\"taxpayerPhone\":\"02084110129\",\"taxpayerBank\":\"1111111111\",\"taxpayerAccount\":\"888888888888888\",\"areaId\":null,\"createTime\":\"2023-09-14 15:08:48\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"91110108MA004CPN95\",\"enterpriseNumbers\":\"91110108MA004CPN95\",\"ompDeptId\":\"862196724060536832\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":37,\"parentId\":1,\"name\":\"9111112222333344DD\",\"code\":\"1A37A\",\"level\":2,\"taxpayerCode\":\"9111112222333344DD\",\"taxpayerName\":null,\"taxpayerAddress\":\"广州市番禺区大学城外环东路132号北学院楼B栋305室\",\"taxpayerPhone\":\"02084110129\",\"taxpayerBank\":\"9111112222333344DD\",\"taxpayerAccount\":\"9111112222333344\",\"areaId\":null,\"createTime\":\"2023-09-14 15:09:13\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"9111112222333344DD\",\"enterpriseNumbers\":\"9111112222333344DD\",\"ompDeptId\":\"862196829375315968\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":38,\"parentId\":1,\"name\":\"申报测试使用税号\",\"code\":\"1A38A\",\"level\":2,\"taxpayerCode\":\"911327300000000013\",\"taxpayerName\":null,\"taxpayerAddress\":\"北京市海淀区\",\"taxpayerPhone\":\"12145678906\",\"taxpayerBank\":\"开户银行\",\"taxpayerAccount\":\"121212121212121212\",\"areaId\":11,\"createTime\":\"2023-09-20 14:21:01\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"911327300000000013\",\"enterpriseNumbers\":\"911327300\",\"ompDeptId\":\"864359027514163200\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":39,\"parentId\":1,\"name\":\"91370522MA3U01A9XQ\",\"code\":\"1A39A\",\"level\":2,\"taxpayerCode\":\"91370522MA3U01A9XQ\",\"taxpayerName\":null,\"taxpayerAddress\":\"121212\",\"taxpayerPhone\":\"1212121\",\"taxpayerBank\":\"1212121\",\"taxpayerAccount\":\"3213212\",\"areaId\":null,\"createTime\":\"2023-09-20 16:00:41\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"91370522MA3U01A9XQ\",\"enterpriseNumbers\":\"91370522MA3U01A9XQ\",\"ompDeptId\":\"864384107136827392\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":40,\"parentId\":1,\"name\":\"916543000000000018\",\"code\":\"1A40A\",\"level\":2,\"taxpayerCode\":\"916543000000000018\",\"taxpayerName\":null,\"taxpayerAddress\":\"916543000000000018\",\"taxpayerPhone\":\"916543000000000018\",\"taxpayerBank\":\"916543000000000018\",\"taxpayerAccount\":\"916543000000000018\",\"areaId\":null,\"createTime\":\"2023-09-20 16:01:31\",\"deptType\":2,\"createUser\":1,\"contactName\":\"916543000000000018\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"916543\",\"enterpriseNumbers\":\"916543000000000018\",\"ompDeptId\":\"864384318789795840\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":41,\"parentId\":1,\"name\":\"91441283MA54JLA8X0\",\"code\":\"1A41A\",\"level\":2,\"taxpayerCode\":\"91441283MA54JLA8X0\",\"taxpayerName\":null,\"taxpayerAddress\":\"91441283MA54JLA8X0\",\"taxpayerPhone\":\"121212\",\"taxpayerBank\":\"91441283MA54JLA8X0\",\"taxpayerAccount\":\"914412835480\",\"areaId\":15,\"createTime\":\"2023-09-20 16:02:07\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"91441291441283MA54JLA8X0\",\"enterpriseNumbers\":\"91441283MA54JLA8X0\",\"ompDeptId\":\"864384469415641088\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":42,\"parentId\":1,\"name\":\"91440700MA4X40YK69\",\"code\":\"1A42A\",\"level\":2,\"taxpayerCode\":\"91440700MA4X40YK69\",\"taxpayerName\":null,\"taxpayerAddress\":\"91440700MA4X40YK69\",\"taxpayerPhone\":\"1212\",\"taxpayerBank\":\"91440700MA4X40YK69\",\"taxpayerAccount\":\"9144070044069\",\"areaId\":null,\"createTime\":\"2023-09-20 16:02:24\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"91440791440700MA4X40YK69\",\"enterpriseNumbers\":\"91440700MA4X40YK69\",\"ompDeptId\":\"864384541494755328\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":43,\"parentId\":1,\"name\":\"91320205MA2369668A\",\"code\":\"1A43A\",\"level\":2,\"taxpayerCode\":\"91320205MA2369668A\",\"taxpayerName\":null,\"taxpayerAddress\":\"91320205MA2369668A\",\"taxpayerPhone\":\"2141235143\",\"taxpayerBank\":\"91320205MA2369668A\",\"taxpayerAccount\":\"913202052369668\",\"areaId\":11,\"createTime\":\"2023-09-20 16:02:45\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"91320291320205MA2369668A\",\"enterpriseNumbers\":\"91320205MA2369668A\",\"ompDeptId\":\"864384629780660224\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":44,\"parentId\":1,\"name\":\"乐企测试企业\",\"code\":\"1A44A\",\"level\":2,\"taxpayerCode\":\"91442000661493375U\",\"taxpayerName\":null,\"taxpayerAddress\":\"1\",\"taxpayerPhone\":\"4\",\"taxpayerBank\":\"2\",\"taxpayerAccount\":\"3\",\"areaId\":null,\"createTime\":\"2023-11-28 11:15:18\",\"deptType\":2,\"createUser\":1,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"乐企测试企业\",\"enterpriseNumbers\":\"91442000661493375U\",\"ompDeptId\":\"889317051731165184\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null},{\"deptId\":45,\"parentId\":1,\"name\":\"测试企业北京的0\",\"code\":\"1A45A\",\"level\":2,\"taxpayerCode\":\"91110102351212916L\",\"taxpayerName\":null,\"taxpayerAddress\":\"北京\",\"taxpayerPhone\":\"13012345678\",\"taxpayerBank\":\"北京\",\"taxpayerAccount\":\"888\",\"areaId\":11,\"createTime\":\"2023-12-12 11:23:10\",\"deptType\":2,\"createUser\":114,\"contactName\":\"\",\"contactPhone\":\"\",\"contactEmail\":\"\",\"deptSname\":\"测试企业北京\",\"enterpriseNumbers\":\"TTY\",\"ompDeptId\":\"894392461141622784\",\"tenantId\":\"1\",\"ompParentId\":\"0\",\"sourceId\":null}],\"plugininfo\":null,\"version\":\"759839dff93540c1a34de48772c30810\",\"expireMinite\":60,\"expireFreshTime\":1724918961382,\"loginCount\":0,\"lastLoginTime\":\"2024-08-29 10:47:51\",\"boradAuth\":0},\"message\":\"success\"}";

            //log.info("第二个要改的@@@@@@:{}",response);
            //log.info("调大B获取的购方集合:{}",response);
        }
        return bigBResponseDateProcess(response);
    }

    private String bigBResponseDateProcess(String response) {
        JSONObject jsonObject = JSON.parseObject(response);
        String retCode = jsonObject.getString(ConfigureConstant.CODE);
        if (ConfigureConstant.SECCESS_CODE.equals(retCode)) {
            String data = jsonObject.getString("data");
            return data;
        } else {
            log.info("调用大B接口失败：{}",response);
            throw new BaseException("调用大B接口失败",402);
        }
    }

    @Override
    public String getUserMenus(String requestToken, String systemSign) {
        Map<String,String> headers = getHeader(requestToken);
        Map<String, String> reqMap = new HashMap<>();
        reqMap.put(ConfigureConstant.BIGB_SYSTEMSIGN, systemSign);
        String bigBResp = "";
        if(Arrays.asList(environment.getActiveProfiles()).contains("dev")){
            bigBResp = "{\"code\":\"0000\",\"data\":[{\"menuId\":800138,\"parentId\":800003,\"name\":\"菜单管理\",\"url\":\"http://10.1.2.221/jxpt/#/menumanage\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/sysmanage/menumanage/index\",\"puserId\":null},{\"menuId\":800106,\"parentId\":800033,\"name\":\"修改业务单元\",\"url\":\"http://10.1.2.221/jxpt/#/updateBussiness\",\"perms\":null,\"type\":3,\"icon\":null,\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"/updateBussiness\",\"puserId\":null},{\"menuId\":800141,\"parentId\":800003,\"name\":\"进项消息列表\",\"url\":\"http://10.1.2.221/jxpt/#/news/list\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":null,\"puserId\":null},{\"menuId\":800011,\"parentId\":800008,\"name\":\"手工勾选\",\"url\":\"http://10.1.2.221/jxpt/#/manualCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/manualCheck/index\",\"puserId\":null},{\"menuId\":800054,\"parentId\":800050,\"name\":\"查验历史\",\"url\":\"http://10.1.2.221/jxpt/#/historyCheck\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/historyCheck/index\",\"puserId\":null},{\"menuId\":800003,\"parentId\":800115,\"name\":\"系统管理\",\"url\":\"http://10.1.2.221/jxpt/#/sysmanage\",\"perms\":\"\",\"type\":1,\"icon\":\"icon_statisticQuery\",\"orderNum\":13,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/index\",\"puserId\":null},{\"menuId\":800101,\"parentId\":800002,\"name\":\"概览\",\"url\":\"http://10.1.2.221/jxpt/#/tickePoolOutline\",\"perms\":null,\"type\":2,\"icon\":\"icon_index_good\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/main\",\"puserId\":null},{\"menuId\":800045,\"parentId\":800034,\"name\":\"发票签收失败报表\",\"url\":\"http://10.1.2.221/jxpt/#/receiptinvoicefailstatistics\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":11,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/receiptinvoicefailstatistics/index\",\"puserId\":null},{\"menuId\":800097,\"parentId\":800095,\"name\":\"实名客票\",\"url\":\"http://10.1.2.221/jxpt/#/realNameInvoice\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/JXTickePool/realNameInvoice/index\",\"puserId\":null},{\"menuId\":800103,\"parentId\":800005,\"name\":\"申请导出\",\"url\":\"http://10.1.2.221/jxpt/#/indexlist\",\"perms\":null,\"type\":3,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":null,\"puserId\":null},{\"menuId\":900329,\"parentId\":800008,\"name\":\"代扣代缴认证\",\"url\":\"http://10.1.2.221/jxpt/#/withholdRemitAuthentication\",\"perms\":\"\",\"type\":2,\"icon\":\"\",\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-11-27 14:52:46\",\"remark\":\"jxpt/invoicecertification/withholdRemitAuthentication/index\",\"puserId\":null},{\"menuId\":800060,\"parentId\":800034,\"name\":\"认证人员统计\",\"url\":\"http://10.1.2.221/jxpt/#/certifyingPersonnel\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":13,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/certifyingPersonnel/index\",\"puserId\":null},{\"menuId\":800038,\"parentId\":800034,\"name\":\"异常发票报表\",\"url\":\"http://10.1.2.221/jxpt/#/exceptionalInvoiceReport\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/exceptionalInvoiceReport/index\",\"puserId\":null},{\"menuId\":800009,\"parentId\":800008,\"name\":\"企业税务信息\",\"url\":\"http://10.1.2.221/jxpt/#/enterpriseTaxInformation\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/enterpriseTaxInformation/index\",\"puserId\":null},{\"menuId\":800004,\"parentId\":800001,\"name\":\"票据采集\",\"url\":\"http://10.1.2.221/jxpt/#/invoicecollection\",\"perms\":null,\"type\":1,\"icon\":\"icon_Authentication\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800118,\"parentId\":800117,\"name\":\"供应商风险\",\"url\":\"http://10.1.2.221/jxpt/#/gysRisk\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/invoiceRisk/gysRisk/index\",\"puserId\":null},{\"menuId\":800080,\"parentId\":800076,\"name\":\"缴款书认证查询\",\"url\":\"http://10.1.2.221/jxpt/#/deductionJksSearch\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/deductionCertification/jksSearch/index\",\"puserId\":null},{\"menuId\":800091,\"parentId\":800069,\"name\":\"缴款书手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundJksManual\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/jksManual/index\",\"puserId\":null},{\"menuId\":900315,\"parentId\":800076,\"name\":\"农产品缴款书加计扣除\",\"url\":\"http://10.1.2.221/jxpt/#/deductionJksSuper\",\"perms\":\"\",\"type\":2,\"icon\":\"\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-10-20 15:52:43\",\"remark\":\"jxpt/deductionCertification/jksSuperDeduction/index\",\"puserId\":null},{\"menuId\":800027,\"parentId\":800025,\"name\":\"扫码签收\",\"url\":\"http://10.1.2.221/jxpt/#/sweepcode\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/sweepcode/index\",\"puserId\":null},{\"menuId\":800081,\"parentId\":800034,\"name\":\"海关缴款书综合查询\",\"url\":\"http://10.1.2.221/jxpt/#/hgjksQuery\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":14,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/hgjksQuery/index\",\"puserId\":null},{\"menuId\":800095,\"parentId\":800002,\"name\":\"进项票池\",\"url\":\"http://10.1.2.221/jxpt/#/JXTickePool\",\"perms\":null,\"type\":1,\"icon\":\"icon_pcjxfp\",\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/index\",\"puserId\":null},{\"menuId\":800107,\"parentId\":800033,\"name\":\"取消签收\",\"url\":\"http://10.1.2.221/jxpt/#/cancelSign\",\"perms\":null,\"type\":3,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"/cancelSign\",\"puserId\":null},{\"menuId\":800012,\"parentId\":800008,\"name\":\"导入勾选\",\"url\":\"http://10.1.2.221/jxpt/#/importCheckChecklist\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/importCheck/index\",\"puserId\":null},{\"menuId\":800023,\"parentId\":800021,\"name\":\"已转出进项税查询\",\"url\":\"http://10.1.2.221/jxpt/#/searchTransferOuted\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/transferOut/searchTransferOuted/index\",\"puserId\":null},{\"menuId\":800117,\"parentId\":800002,\"name\":\"票据风险\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceRisk\",\"perms\":null,\"type\":1,\"icon\":\"icon_pjfx\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/index\",\"puserId\":null},{\"menuId\":800047,\"parentId\":800001,\"name\":\"监控预警\",\"url\":\"http://10.1.2.221/jxpt/#/monitearly\",\"perms\":null,\"type\":1,\"icon\":\"icon_warning\",\"orderNum\":10,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800082,\"parentId\":800034,\"name\":\"转内销发票查询\",\"url\":\"http://10.1.2.221/jxpt/#/znxInvoiceSearch\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":15,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/znxInvoiceSearch/index\",\"puserId\":null},{\"menuId\":800140,\"parentId\":800003,\"name\":\"字典管理\",\"url\":\"http://10.1.2.221/jxpt/#/dictionary\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/sysmanage/dictionary/index\",\"puserId\":null},{\"menuId\":800065,\"parentId\":800004,\"name\":\"异常海关缴款书\",\"url\":\"http://10.1.2.221/jxpt/#/unfilledhgjks\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecollection/unfilledhgjks/index\",\"puserId\":null},{\"menuId\":800126,\"parentId\":800034,\"name\":\"发票明细查询\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceDetailSearch\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":27,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/invoiceDetailSearch/index\",\"puserId\":null},{\"menuId\":800051,\"parentId\":800050,\"name\":\"扫码查验\",\"url\":\"http://10.1.2.221/jxpt/#/scanCodeCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/scanCodeCheck/index\",\"puserId\":null},{\"menuId\":800074,\"parentId\":800069,\"name\":\"缴款书认证查询\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundJksSearch\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/jksSearch/index\",\"puserId\":null},{\"menuId\":800073,\"parentId\":800069,\"name\":\"缴款书导入认证\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundImportcertification\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/jksImportcertification/index\",\"puserId\":null},{\"menuId\":800059,\"parentId\":800050,\"name\":\"综合查询\",\"url\":\"http://10.1.2.221/jxpt/#/checkComprehensiveQuery\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/checkComprehensiveQuery/index\",\"puserId\":null},{\"menuId\":800084,\"parentId\":800001,\"name\":\"出口转内销认证\",\"url\":\"http://10.1.2.221/jxpt/#/exportTodomestic\",\"perms\":null,\"type\":1,\"icon\":\"icon_znx\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800015,\"parentId\":800008,\"name\":\"扫码认证\",\"url\":\"http://10.1.2.221/jxpt/#/scancode\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":9,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/scancode/index\",\"puserId\":null},{\"menuId\":800071,\"parentId\":800069,\"name\":\"发票导入认证\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceImportcertification\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/importcertification/index\",\"puserId\":null},{\"menuId\":800020,\"parentId\":800008,\"name\":\"设置不抵扣\",\"url\":\"http://10.1.2.221/jxpt/#/noCheck\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":14,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/noCheck/index\",\"puserId\":null},{\"menuId\":800044,\"parentId\":800034,\"name\":\"认证发票汇总报表\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceauthenticationsummarystatistics\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":9,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/invoiceauthenticationsummarystatistics/index\",\"puserId\":null},{\"menuId\":800113,\"parentId\":800087,\"name\":\"撤销认证\",\"url\":\"http://10.1.2.221/jxpt/#RevokenoCheck\",\"perms\":null,\"type\":3,\"icon\":null,\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"RevokenoCheck\",\"puserId\":null},{\"menuId\":800137,\"parentId\":800003,\"name\":\"企业接入管理\",\"url\":\"http://10.1.2.221/jxpt/#/accessManagement\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/sysmanage/accessManagement/index\",\"puserId\":null},{\"menuId\":800049,\"parentId\":800047,\"name\":\"普票跨年度预警\",\"url\":\"http://10.1.2.221/jxpt/#/definitiveLateyeaRearly\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/monitearly/definitiveLateyeaRearly/index\",\"puserId\":null},{\"menuId\":800128,\"parentId\":800017,\"name\":\"异常发票撤销\",\"url\":\"http://10.1.2.221/jxpt/#ycfpcxfn\",\"perms\":null,\"type\":3,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"ycfpcxfn\",\"puserId\":null},{\"menuId\":800037,\"parentId\":800034,\"name\":\"异常发票查询\",\"url\":\"http://10.1.2.221/jxpt/#/exceptionalInvoiceQuery\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/exceptionalInvoiceQuery/index\",\"puserId\":null},{\"menuId\":800005,\"parentId\":800004,\"name\":\"发票采集列表\",\"url\":\"http://10.1.2.221/jxpt/#/indexlist\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecollection/indexlist/index\",\"puserId\":null},{\"menuId\":800094,\"parentId\":800092,\"name\":\"销项采集\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceCollectionXX\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/invoiceCollection/invoiceCollectionXX/index\",\"puserId\":null},{\"menuId\":800098,\"parentId\":800095,\"name\":\"其他票据\",\"url\":\"http://10.1.2.221/jxpt/#/otherInvoice\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/JXTickePool/otherInvoice/index\",\"puserId\":null},{\"menuId\":800114,\"parentId\":800050,\"name\":\"电票查验\",\"url\":\"http://10.1.2.221/jxpt/#/einvoiceCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/einvoiceCheck/index\",\"puserId\":null},{\"menuId\":800102,\"parentId\":800001,\"name\":\"首页\",\"url\":\"http://10.1.2.221/jxpt/#/tickeHome\",\"perms\":null,\"type\":1,\"icon\":\"icon_index_good\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/dashboard/index\",\"puserId\":null},{\"menuId\":800010,\"parentId\":800008,\"name\":\"扫码勾选\",\"url\":\"http://10.1.2.221/jxpt/#/scanCheckChecklist\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/scanCheck/index\",\"puserId\":null},{\"menuId\":800064,\"parentId\":800004,\"name\":\"海关缴款书录入\",\"url\":\"http://10.1.2.221/jxpt/#/hgjksEntry\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecollection/hgjksEntry/index\",\"puserId\":null},{\"menuId\":800055,\"parentId\":800050,\"name\":\"统计查询\",\"url\":\"http://10.1.2.221/jxpt/#/statisticsCheck\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/statisticsCheck/index\",\"puserId\":null},{\"menuId\":800099,\"parentId\":800002,\"name\":\"销项票池\",\"url\":\"http://10.1.2.221/jxpt/#/XXTickePool\",\"perms\":null,\"type\":1,\"icon\":\"icon_pcxxfp\",\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/index\",\"puserId\":null},{\"menuId\":800043,\"parentId\":800034,\"name\":\"发票签收统计表\",\"url\":\"http://10.1.2.221/jxpt/#/receiptStatistics\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":10,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/receiptStatistics/index\",\"puserId\":null},{\"menuId\":800063,\"parentId\":800004,\"name\":\"海关缴款书采集\",\"url\":\"http://10.1.2.221/jxpt/#/hgjksCollection\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecollection/hgjksCollection/index\",\"puserId\":null},{\"menuId\":800072,\"parentId\":800069,\"name\":\"发票认证查询\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundInvoiceSearch\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/invoiceSearch/index\",\"puserId\":null},{\"menuId\":800067,\"parentId\":800025,\"name\":\"缴款书导入签收\",\"url\":\"http://10.1.2.221/jxpt/#/jksimportSign\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":12,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/jksimportSign/index\",\"puserId\":null},{\"menuId\":800017,\"parentId\":800008,\"name\":\"认证查询\",\"url\":\"http://10.1.2.221/jxpt/#/search\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":11,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/search/index\",\"puserId\":null},{\"menuId\":800104,\"parentId\":800003,\"name\":\"显示列表管理\",\"url\":\"http://10.1.2.221/jxpt/#/listDistribution\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":10,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/sysmanage/listDistribution/index\",\"puserId\":null},{\"menuId\":800002,\"parentId\":0,\"name\":\"企业票池\",\"url\":\"http://10.1.2.221/jxpt/#/\",\"perms\":null,\"type\":0,\"icon\":\"icon_qypc\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"\",\"puserId\":null},{\"menuId\":800379,\"parentId\":800377,\"name\":\"缴款书入账\",\"url\":\"http://10.1.2.221/jxpt/#/paymentBookEntry\",\"perms\":null,\"type\":2,\"icon\":\"icon_index_good\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/entry/paymentBookEntry/index\",\"puserId\":null},{\"menuId\":800068,\"parentId\":800025,\"name\":\"缴款书签收查询\",\"url\":\"http://10.1.2.221/jxpt/#/jksSignininquery\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":13,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/jksSignininquery/index\",\"puserId\":null},{\"menuId\":800034,\"parentId\":800001,\"name\":\"统计查询\",\"url\":\"http://10.1.2.221/jxpt/#/report\",\"perms\":null,\"type\":1,\"icon\":\"icon_statisticsReportform\",\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800122,\"parentId\":800120,\"name\":\"签收日志\",\"url\":\"http://10.1.2.221/jxpt/#/signinLog\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/invoiceSigninXX/signinLog/index\",\"puserId\":null},{\"menuId\":800026,\"parentId\":800025,\"name\":\"手工签收\",\"url\":\"http://10.1.2.221/jxpt/#/handwork\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/handwork/index\",\"puserId\":null},{\"menuId\":800053,\"parentId\":800050,\"name\":\"导入查验\",\"url\":\"http://10.1.2.221/jxpt/#/importCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/importCheck/index\",\"puserId\":null},{\"menuId\":800105,\"parentId\":800033,\"name\":\"批量签收\",\"url\":\"http://10.1.2.221/jxpt/#/updateQs\",\"perms\":null,\"type\":3,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"/updateQs\",\"puserId\":null},{\"menuId\":800076,\"parentId\":800001,\"name\":\"缴款书认证\",\"url\":\"http://10.1.2.221/jxpt/#/deductionCertification\",\"perms\":null,\"type\":1,\"icon\":\"icon_hgjks\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800031,\"parentId\":800025,\"name\":\"扫描仪签收\",\"url\":\"http://10.1.2.221/jxpt/#/scaninvoice\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/scaninvoice/index\",\"puserId\":null},{\"menuId\":800088,\"parentId\":800084,\"name\":\"缴款书手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/znxJksManual\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/exportTodomestic/jksManual/index\",\"puserId\":null},{\"menuId\":800077,\"parentId\":800076,\"name\":\"缴款书手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/deductionJksManual\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/deductionCertification/jksManual/index\",\"puserId\":null},{\"menuId\":800018,\"parentId\":800008,\"name\":\"抵扣统计\",\"url\":\"http://10.1.2.221/jxpt/#/witnessing\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":12,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/witnessing/index\",\"puserId\":null},{\"menuId\":800124,\"parentId\":800123,\"name\":\"销项汇总表（按税率）\",\"url\":\"http://10.1.2.221/jxpt/#/taxRateStatement\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/declarationData/taxRateStatement/index\",\"puserId\":null},{\"menuId\":800025,\"parentId\":800001,\"name\":\"票据签收\",\"url\":\"http://10.1.2.221/jxpt/#/signin\",\"perms\":null,\"type\":1,\"icon\":\"icon_sign\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800024,\"parentId\":800021,\"name\":\"取消已转出进项税\",\"url\":\"http://10.1.2.221/jxpt/#/cancelTransferOut\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/transferOut/cancelTransferOut/index\",\"puserId\":null},{\"menuId\":800377,\"parentId\":800001,\"name\":\"票据入账\",\"url\":\"http://10.1.2.221/jxpt/#/entry\",\"perms\":null,\"type\":1,\"icon\":\"icon_sign\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800120,\"parentId\":800002,\"name\":\"销项发票签收\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceSigninXX\",\"perms\":null,\"type\":1,\"icon\":\"icon_pccj\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/index\",\"puserId\":null},{\"menuId\":800042,\"parentId\":800034,\"name\":\"认证结果清单\",\"url\":\"http://10.1.2.221/jxpt/#/authResultList\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/authResultList/index\",\"puserId\":null},{\"menuId\":900286,\"parentId\":800008,\"name\":\"农产品发票加计扣除\",\"url\":\"http://10.1.2.221/jxpt/#/superDeduction\",\"perms\":\"\",\"type\":2,\"icon\":\"\",\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-10-18 10:08:21\",\"remark\":\"jxpt/invoicecertification/superDeduction/index\",\"puserId\":null},{\"menuId\":800070,\"parentId\":800069,\"name\":\"发票手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundInvoiceManual\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/invoiceManual/index\",\"puserId\":null},{\"menuId\":800008,\"parentId\":800001,\"name\":\"发票认证\",\"url\":\"http://10.1.2.221/jxpt/#/invoicecertification\",\"perms\":null,\"type\":1,\"icon\":\"icon_invoicecertification\",\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800108,\"parentId\":800033,\"name\":\"导出图片\",\"url\":\"http://10.1.2.221/jxpt/#/downImage\",\"perms\":null,\"type\":3,\"icon\":null,\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"/downImage\",\"puserId\":null},{\"menuId\":800139,\"parentId\":800003,\"name\":\"定时任务\",\"url\":\"http://10.1.2.221/jxpt/#/crontab\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/sysmanage/crontab/index\",\"puserId\":null},{\"menuId\":800083,\"parentId\":800034,\"name\":\"转内销缴款书查询\",\"url\":\"http://10.1.2.221/jxpt/#/znxJksSearch\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":16,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/znxJksSearch/index\",\"puserId\":null},{\"menuId\":800062,\"parentId\":800034,\"name\":\"发票扫描明细\",\"url\":\"http://10.1.2.221/jxpt/#/signininqueryMRYX\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":18,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/signininqueryMRYX/index\",\"puserId\":null},{\"menuId\":800096,\"parentId\":800095,\"name\":\"增值税发票\",\"url\":\"http://10.1.2.221/jxpt/#/valueAddedTicke\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/JXTickePool/valueAddedTicke/index\",\"puserId\":null},{\"menuId\":800121,\"parentId\":800120,\"name\":\"签收查询\",\"url\":\"http://10.1.2.221/jxpt/#/signinQuery\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/invoiceSigninXX/signinQuery/index\",\"puserId\":null},{\"menuId\":800116,\"parentId\":800025,\"name\":\"电票签收\",\"url\":\"http://10.1.2.221/jxpt/#/upload\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/einvoice/upload/index\",\"puserId\":null},{\"menuId\":800087,\"parentId\":800084,\"name\":\"发票认证查询\",\"url\":\"http://10.1.2.221/jxpt/#/znxInvoiceRzSearch\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/exportTodomestic/invoiceSearch/index\",\"puserId\":null},{\"menuId\":800056,\"parentId\":800003,\"name\":\"日志管理\",\"url\":\"http://10.1.2.221/jxpt/#/gateLog\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/monitor/gateLog/index\",\"puserId\":null},{\"menuId\":800022,\"parentId\":800021,\"name\":\"进项税转出\",\"url\":\"http://10.1.2.221/jxpt/#/transferOut\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/transferOut/transferOut/index\",\"puserId\":null},{\"menuId\":800089,\"parentId\":800084,\"name\":\"缴款书不抵扣\",\"url\":\"http://10.1.2.221/jxpt/#/znxJksNoCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/exportTodomestic/jksNoCheck/index\",\"puserId\":null},{\"menuId\":800050,\"parentId\":800001,\"name\":\"发票查验\",\"url\":\"http://10.1.2.221/jxpt/#/check\",\"perms\":null,\"type\":1,\"icon\":\"icon_Inspection2\",\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800075,\"parentId\":800069,\"name\":\"退税统计\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundWitnessing\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/taxRefundCertification/taxRefundWitnessing/index\",\"puserId\":null},{\"menuId\":800035,\"parentId\":800034,\"name\":\"发票综合查询\",\"url\":\"http://10.1.2.221/jxpt/#/comprehensiveInvoiceQuery\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/comprehensiveInvoiceQuery/index\",\"puserId\":null},{\"menuId\":800040,\"parentId\":800034,\"name\":\"发票认证日报\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceAuthDailyReport\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/invoiceAuthDailyReport/index\",\"puserId\":null},{\"menuId\":800078,\"parentId\":800076,\"name\":\"缴款书导入认证\",\"url\":\"http://10.1.2.221/jxpt/#/deductionImportcertification\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/deductionCertification/jksImportcertification/index\",\"puserId\":null},{\"menuId\":800033,\"parentId\":800025,\"name\":\"签收查询\",\"url\":\"http://10.1.2.221/jxpt/#/signininquery\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/signininquery/index\",\"puserId\":null},{\"menuId\":800069,\"parentId\":800001,\"name\":\"退税认证\",\"url\":\"http://10.1.2.221/jxpt/#/taxRefundCertification\",\"perms\":null,\"type\":1,\"icon\":\"icon_ts\",\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null},{\"menuId\":800013,\"parentId\":800008,\"name\":\"勾选发票确认\",\"url\":\"http://10.1.2.221/jxpt/#/confirmationCheck\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/confirmationCheck/index\",\"puserId\":null},{\"menuId\":800092,\"parentId\":800002,\"name\":\"票据采集\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceCollection\",\"perms\":null,\"type\":1,\"icon\":\"icon_pccj\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/index\",\"puserId\":null},{\"menuId\":800109,\"parentId\":800033,\"name\":\"批量备注\",\"url\":\"http://10.1.2.221/jxpt/#/changeRemark\",\"perms\":null,\"type\":3,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"/changeRemark\",\"puserId\":null},{\"menuId\":800111,\"parentId\":800017,\"name\":\"撤销认证\",\"url\":\"http://10.1.2.221/jxpt/#cxrzFp\",\"perms\":null,\"type\":3,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"cxrzFp\",\"puserId\":null},{\"menuId\":800007,\"parentId\":800004,\"name\":\"未补明细发票\",\"url\":\"http://10.1.2.221/jxpt/#/unfilledlist\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecollection/unfilledlist/index\",\"puserId\":null},{\"menuId\":800090,\"parentId\":800084,\"name\":\"缴款书认证查询\",\"url\":\"http://10.1.2.221/jxpt/#/jksSearch\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":6,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/exportTodomestic/jksSearch/index\",\"puserId\":null},{\"menuId\":800378,\"parentId\":800377,\"name\":\"发票入账\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceEntry\",\"perms\":null,\"type\":2,\"icon\":\"icon_index_good\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/entry/invoiceEntry/index\",\"puserId\":null},{\"menuId\":800123,\"parentId\":800002,\"name\":\"增值税申报数据\",\"url\":\"http://10.1.2.221/jxpt/#/declarationData\",\"perms\":null,\"type\":1,\"icon\":\"icon_pjfx\",\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/index\",\"puserId\":null},{\"menuId\":800014,\"parentId\":800008,\"name\":\"手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/manual\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/manual/index\",\"puserId\":null},{\"menuId\":800041,\"parentId\":800034,\"name\":\"发票认证月报\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceAuthMonthlyReport\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/invoiceAuthMonthlyReport/index\",\"puserId\":null},{\"menuId\":800119,\"parentId\":800117,\"name\":\"风险发票\",\"url\":\"http://10.1.2.221/jxpt/#/riskInvoice\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/invoiceRisk/riskInvoice/index\",\"puserId\":null},{\"menuId\":800110,\"parentId\":800058,\"name\":\"一键认证\",\"url\":\"http://10.1.2.221/jxpt/#/fpxxrzAll\",\"perms\":null,\"type\":3,\"icon\":null,\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"fpxxrzAll\",\"puserId\":null},{\"menuId\":800112,\"parentId\":800080,\"name\":\"撤销认证\",\"url\":\"http://10.1.2.221/jxpt/#RevokenoCheck\",\"perms\":null,\"type\":3,\"icon\":null,\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"RevokenoCheck\",\"puserId\":null},{\"menuId\":800086,\"parentId\":800084,\"name\":\"发票不抵扣\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceNoCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/exportTodomestic/invoiceNoCheck/index\",\"puserId\":null},{\"menuId\":800066,\"parentId\":800025,\"name\":\"缴款书手工签收\",\"url\":\"http://10.1.2.221/jxpt/#/jkshandwork\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":11,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/jkshandwork/index\",\"puserId\":null},{\"menuId\":800061,\"parentId\":800021,\"name\":\"转出原因配置\",\"url\":\"http://10.1.2.221/jxpt/#/outReasonConfig\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/transferOut/outReasonConfig/index\",\"puserId\":null},{\"menuId\":800001,\"parentId\":0,\"name\":\"进项系统\",\"url\":\"http://10.1.2.221/jxpt/#/\",\"perms\":null,\"type\":0,\"icon\":\"icon_jxpt\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"\",\"puserId\":null},{\"menuId\":800079,\"parentId\":800076,\"name\":\"缴款书设置不抵扣\",\"url\":\"http://10.1.2.221/jxpt/#/deductionNoCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":3,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/deductionCertification/jksNoCheck/index\",\"puserId\":null},{\"menuId\":800048,\"parentId\":800047,\"name\":\"异常发票预警\",\"url\":\"http://10.1.2.221/jxpt/#/abnormalInvoiceEarly\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/monitearly/abnormalInvoiceEarly/index\",\"puserId\":null},{\"menuId\":800006,\"parentId\":800004,\"name\":\"异常发票采集\",\"url\":\"http://10.1.2.221/jxpt/#/abnormal\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecollection/abnormal/index\",\"puserId\":null},{\"menuId\":800052,\"parentId\":800050,\"name\":\"扫描查验\",\"url\":\"http://10.1.2.221/jxpt/#/scanCheck\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/check/scanCheck/index\",\"puserId\":null},{\"menuId\":800016,\"parentId\":800008,\"name\":\"导入认证\",\"url\":\"http://10.1.2.221/jxpt/#/importcertification\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":10,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/importcertification/index\",\"puserId\":null},{\"menuId\":800127,\"parentId\":800095,\"name\":\"海关缴款书\",\"url\":\"http://10.1.2.221/jxpt/#/valueAddedTickeJks\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/JXTickePool/valueAddedTickeJks/index\",\"puserId\":null},{\"menuId\":800125,\"parentId\":800123,\"name\":\"销项汇总表（按项目）\",\"url\":\"http://10.1.2.221/jxpt/#/projectReport\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":2,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/declarationData/projectReport/index\",\"puserId\":null},{\"menuId\":800100,\"parentId\":800099,\"name\":\"增值税发票\",\"url\":\"http://10.1.2.221/jxpt/#/XXValueAddedTicke\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":0,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/XXTickePool/XXValueAddedTicke/index\",\"puserId\":null},{\"menuId\":800057,\"parentId\":800008,\"name\":\"智能勾选\",\"url\":\"http://10.1.2.221/jxpt/#/IntelligentSelection\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":5,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/IntelligentSelection/index\",\"puserId\":null},{\"menuId\":800115,\"parentId\":0,\"name\":\"运维系统\",\"url\":\"http://10.1.2.221/jxpt/#/\",\"perms\":null,\"type\":0,\"icon\":\"icon_statisticQuery\",\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"\",\"puserId\":null},{\"menuId\":800136,\"parentId\":800003,\"name\":\"基础设置\",\"url\":\"http://10.1.2.221/jxpt/#/basicConfiguration\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"operationManage/sysmanage/basicConfiguration/index\",\"puserId\":null},{\"menuId\":800046,\"parentId\":800034,\"name\":\"已签收未认证报表\",\"url\":\"http://10.1.2.221/jxpt/#/signeduncertified\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":12,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/signeduncertified/index\",\"puserId\":null},{\"menuId\":800019,\"parentId\":800008,\"name\":\"撤销抵扣统计\",\"url\":\"http://10.1.2.221/jxpt/#/deductionStatistics\",\"perms\":null,\"type\":2,\"icon\":\"\",\"orderNum\":13,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/deductionStatistics/index\",\"puserId\":null},{\"menuId\":800032,\"parentId\":800025,\"name\":\"导入签收\",\"url\":\"http://10.1.2.221/jxpt/#/importSign\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/signin/importSign/index\",\"puserId\":null},{\"menuId\":900270,\"parentId\":800008,\"name\":\"农产品发票确认\",\"url\":\"http://10.1.2.221/jxpt/#/confirmationAgriculturalProducts\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":7,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/confirmationAgriculturalProducts/index\",\"puserId\":null},{\"menuId\":800093,\"parentId\":800092,\"name\":\"进项采集\",\"url\":\"http://10.1.2.221/jxpt/#/invoiceCollectionJX\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"corporateTickePool/invoiceCollection/invoiceCollectionJX/index\",\"puserId\":null},{\"menuId\":800058,\"parentId\":800008,\"name\":\"签收手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/signManual\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":8,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/invoicecertification/signManual/index\",\"puserId\":null},{\"menuId\":800085,\"parentId\":800084,\"name\":\"发票手工认证\",\"url\":\"http://10.1.2.221/jxpt/#/znxInvoiceManual\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/exportTodomestic/invoiceManual/index\",\"puserId\":null},{\"menuId\":800039,\"parentId\":800034,\"name\":\"进项税额报表\",\"url\":\"http://10.1.2.221/jxpt/#/inputTaxReport\",\"perms\":null,\"type\":2,\"icon\":null,\"orderNum\":4,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/report/inputTaxReport/index\",\"puserId\":null},{\"menuId\":800135,\"parentId\":800090,\"name\":\"撤销认证\",\"url\":\"http://10.1.2.221/jxpt/#RevokenoCheck\",\"perms\":\"\",\"type\":3,\"icon\":\"\",\"orderNum\":1,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"\",\"puserId\":null},{\"menuId\":800021,\"parentId\":800001,\"name\":\"进项税转出\",\"url\":\"http://10.1.2.221/jxpt/#/transfer\",\"perms\":null,\"type\":1,\"icon\":\"icon_turnOut\",\"orderNum\":11,\"systemSign\":\"JXXT\",\"createTime\":\"2023-07-17 12:14:04\",\"remark\":\"jxpt/index\",\"puserId\":null}],\"message\":\"success\"}";
        }else{
            bigBResp = HttpUtils.doPostWithHeader(userCenterConfig.getUserMenuListUrl()
                    , JsonUtils.getInstance().toJsonString(reqMap), headers);
            //log.info("大B返回的菜单:{}",bigBResp);
        }

        return bigBResponseDateProcess(bigBResp);
    }

    @Override
    public void saveUserInfo(String requestToken) {
        BigBUserInfoEntity bigBUserInfo = JsonUtils.getInstance().fromJson(getUserBasicInfo(requestToken), BigBUserInfoEntity.class);
        UserInfo userInfo = changeUserInfo(bigBUserInfo);
        BaseContextHandler.setLoginname(bigBUserInfo.getUsername());
        BaseContextHandler.setUsername(bigBUserInfo.getName());
        BaseContextHandler.setUserID(String.valueOf(bigBUserInfo.getUserId()));
        BaseContextHandler.setUserInfo(JSON.toJSONString(userInfo));
        BaseContextHandler.setToken(requestToken);

    }


    private UserInfo changeUserInfo(BigBUserInfoEntity bigBUserInfo) {
        DynamicContextHolder.push(DbConstant.BASICS_READ);
        UserInfo userInfo = new UserInfo();
        userInfo.setUsername(bigBUserInfo.getName());
        userInfo.setLoginname(bigBUserInfo.getUsername());
        userInfo.setPassword("");
        userInfo.setId(String.valueOf(bigBUserInfo.getUserId()));
        userInfo.setCompany(StringUtils.isBlank(bigBUserInfo.getCompanyCode())?ConfigureConstant.ZXQY :bigBUserInfo.getCompanyCode());
        userInfo.setCompanyName(StringUtils.isBlank(bigBUserInfo.getCompanyName())?ConfigureConstant.ZXQY_HZ :bigBUserInfo.getCompanyName());
//        userInfo.setWebscoketUrl(webscoketUrl);
        SysGroupConfigEntity sce = sysGroupConfigAuthDao.selectOne(new QueryWrapper<SysGroupConfigEntity>().eq("company", StringUtils.isBlank(bigBUserInfo.getCompanyCode())?ConfigureConstant.ZXQY :bigBUserInfo.getCompanyCode()));
        List<Business> blist = new ArrayList<>();
        if (sce != null) {
            userInfo.setJtpz(sce);
            userInfo.setQsStatus(StringUtils.isNotBlank(sce.getQsStatus())?sce.getQsStatus():ConfigureConstant.STRING_1);
            userInfo.setDbName(StringUtils.isNotBlank(sce.getDbName())?sce.getDbName():ConfigureConstant.DEFAULT_DB_NAME);

            if (sce!=null && ConfigureConstant.STRING_1.equals(sce.getBusinessEnable())) {
                //设置业务类型
                String deptType = ConfigureConstant.STRING_4;
                if (ConfigureConstant.STRING_0.equals(sce.getBusinessType())) {
                    deptType = ConfigureConstant.STRING_4;
                } else if (ConfigureConstant.STRING_1.equals(sce.getBusinessType())) {
                    deptType = ConfigureConstant.STRING_5;
                }
                String finalDeptType = deptType;
                List<BigBOrgInfoEntity> businessList = bigBUserInfo.getDeptList().stream().filter(e -> finalDeptType.equals(e.getDeptType())).collect(Collectors.toList());
                businessList.forEach(e->{
                    Business be = new Business();
                    be.setBusinessCode(e.getEnterpriseNumbers());
                    be.setBusinessName(e.getName());
                    blist.add(be);
                });
            }
        }
        userInfo.setBusiness(blist);

        //设置数据权限
        List<BigBOrgInfoEntity> orgList = bigBUserInfo.getDeptList().stream().filter(e -> StringUtils.isNotBlank(e.getTaxpayerCode())).collect(Collectors.toList());
        List<Tax> taxList = new ArrayList<>();
        orgList.forEach(e->{
            Tax te = new Tax();
            te.setTaxno(e.getTaxpayerCode());
            te.setTaxname(e.getName());
            te.setUpgradeStatus("1");
            te.setCompany(StringUtils.isBlank(bigBUserInfo.getCompanyCode())?ConfigureConstant.ZXQY :bigBUserInfo.getCompanyCode());
            taxList.add(te);
        });
        userInfo.setOrg(taxList);
        return userInfo;
    }

    @Override
    public void removeUserInfo() {
        BaseContextHandler.remove();
    }

}
